local xU_Ver="4.3.1"
local xU_Tag="|c0000FFFFxUtils "..xU_Ver.."|r: "
-- xUtils 4.3.1 by Everglow
-- to disable a function change it to false
-- current  functionality:
-------------------------------------------------------------------------
	-- increases the camera zoom-out distance to the maximum
		local maxzoom = true;
	-- moves the edit boxes to the top of the chat frames
		local topedit = true;
	-- closes the edit box if you enter combat while typing in chat (supports WIM)
		local smartedit = true;
	-- autoselect most valuable quest reward
		local autoselectreward = true;
	-- sets your reputation bar to the current faction whenever you gain rep
	-- *NOTE: Will not switch to factions that are inactive or under a header that is collapsed in your Reputaion window
		local smartrep = true;
	-- autodismounts when needed
		local autodismount = true;
	-- autoconfirm greed and/or disenchant rolls
		local autogreed = true;
		local autode = true; 
	-- recycle unused memory whenever you change zones
		local memcycle = true;
		-- you may edit the following paramaters... intvl is specified in seconds
			local intvl = 300; -- the minimum interval between updates in seconds (default is 5 minutes)
			local memReport = true; -- prints a memory report to chat whenever memory is recycled
-------------------------------------------------------------------------
		
-- the following parameters are used by smartedit if it is enabled (do not edit):
	local wim_up, wim_box;
-- the following parameters are used by memcycle if it is enabled (do not edit):
	local lastcycle, lastmem;


function xMsg(msg)
	if msg then
		DEFAULT_CHAT_FRAME:AddMessage(xU_Tag..msg);
	end
end


-- local functions
local function setoptions()
	if maxzoom then 
		SetCVar("CameraDistanceMax", 50);
	end
	if topedit then
		for i = 1, NUM_CHAT_WINDOWS do
			local eb = _G[("ChatFrame%dEditBox"):format(i)]
			eb:ClearAllPoints()
			eb:SetPoint("BOTTOMLEFT",  ("ChatFrame%d"):format(i), "TOPLEFT",  -3, 18)
			eb:SetPoint("BOTTOMRIGHT", ("ChatFrame%d"):format(i), "TOPRIGHT", 3, 18)
		end
	end
	if memcycle then
		collectgarbage();
		lastcycle = time();
		lastmem = collectgarbage("count");
	end
end

function xUtils_onLoad(frame)
	frame:RegisterEvent("PLAYER_LOGIN");
	if smartedit then
		frame:RegisterEvent("PLAYER_REGEN_DISABLED");
		frame:RegisterEvent("PLAYER_REGEN_ENABLED");
	end
	if autoselectreward then
		frame:RegisterEvent("QUEST_COMPLETE");
	end
	if smartrep then
		frame:RegisterEvent("COMBAT_TEXT_UPDATE");
	end
	if autodismount then
		frame:RegisterEvent("UI_ERROR_MESSAGE");
	end
	if autode then
		frame:RegisterEvent("CONFIRM_DISENCHANT_ROLL");
	end
	if autogreed then
		frame:RegisterEvent("CONFIRM_LOOT_ROLL");
	end
	if memcycle then
		frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	end
end
	
function xUtils_OnEvent(frame, event, ...)
	if event == "PLAYER_LOGIN" then
		if maxzoom then 
			SetCVar("CameraDistanceMax", 50);
		end
		if topedit then
			for i = 1, NUM_CHAT_WINDOWS do
				local eb = _G[("ChatFrame%dEditBox"):format(i)]
				eb:ClearAllPoints()
				eb:SetPoint("BOTTOMLEFT",  ("ChatFrame%d"):format(i), "TOPLEFT",  -3, 18)
				eb:SetPoint("BOTTOMRIGHT", ("ChatFrame%d"):format(i), "TOPRIGHT", 3, 18)
			end
		end
		if memcycle then
			collectgarbage();
			lastcycle = time();
			lastmem = collectgarbage("count");
			if memReport then
				xMsg("loaded. Beginning Lua memory in use after all addons loaded: |c00FFFF66"..tostring(math.floor((lastmem/100)+.5)/10).." MB|r");
			end
		end
		
	elseif event == "PLAYER_REGEN_DISABLED" then --player enters combat
		wim_box = WIM_EditBoxInFocus
		if wim_box then
			wim_up = true
			wim_box:ClearFocus()
		else
			ChatFrame1EditBox:ClearFocus()
		end
	elseif event == "PLAYER_REGEN_ENABLED" then	--player leaves combat
		if (wim_up) then
			wim_box:SetFocus()
			wim_up = false
		end

	elseif event == "QUEST_COMPLETE" then -- Quest turn in
		local choices, high, hLink, hID = GetNumQuestChoices(), 0
		if(choices > 1) then
			for i=1, choices do
				local link	= GetQuestItemLink("choice", i);
				local _, _, _, _, _, _, _, _, _, _, value = GetItemInfo(link);
				if(value >= high) then
					high  = value;
					hLink = link;
					hID   = i;
				end
			end
		end
		if(high > 0) then
			local choice = _G["QuestInfoItem"..hID];
			QuestInfoFrame.itemChoice = hID;
			QuestInfoItemHighlight:SetPoint("TOPLEFT", choice, "TOPLEFT", -8, 7);
			QuestInfoItemHighlight:Show();
		end
	
	elseif event == "COMBAT_TEXT_UPDATE" then
		local mType, newFaction = ...;
		if mType ~= "FACTION" then return end
		local curFaction = GetWatchedFactionInfo()
		if(newFaction ~= curFaction) then
			for i = 1, GetNumFactions() do
				if IsFactionInactive(i) then break end;
				local name,_,_,_,_,_,_,_,isHeader,_,hasRep = GetFactionInfo(i)
				if name == newFaction then
					if (isHeader and hasRep) or not isHeader then
						SetWatchedFactionIndex(i)
						xMsg("Reputation bar set to "..newFaction)
					end
					break;
				end
			end		
		end

	elseif event == "UI_ERROR_MESSAGE" then
		local err = ...;
		local adf = GetCVarBool("autoDismountFlying");
		if(err==SPELL_FAILED_NOT_MOUNTED or err==ERR_TAXIPLAYERALREADYMOUNTED) then
			if adf and IsFlying() then
				return;
			else
				Dismount();
			end
		end
	
	elseif event == "CONFIRM_DISENCHANT_ROLL" or event == "CONFIRM_LOOT_ROLL" then
		local rId, rType = ...;	
		if rType ~= 1 then
			for i=1,STATICPOPUP_NUMDIALOGS do
				local popup = _G["StaticPopup"..i]
				if popup.data == rId and popup.data2 == rType and popup:IsVisible() then
					StaticPopup_OnClick(popup, 1)
				end
			end
		end
	
	elseif event == "ZONE_CHANGED_NEW_AREA" then --time to recycle unused memory
		local newtime = time();
		local elapsed = difftime(newtime,lastcycle);
		if elapsed > intvl then
			local before = collectgarbage("count");
			collectgarbage();
			local after = collectgarbage("count");
			if memReport then
				local freed = tostring(math.floor((before-after)+.5));
				local inUse = tostring(math.floor((after/100)+.5)/10);
				local growth = tostring((math.floor((((before-lastmem)/elapsed)*10)+.5)/10));
				xMsg("memory recycled\n"..
					"memory freed: |c00FFFF66"..freed.." kb|r.\n"..
					"current memory use: |c00FFFF66"..inUse.." MB|r.\n"..
					"memory growth rate: |c00FFFF66"..growth.." kb/s|r.");
			end
			lastcycle = newtime;
			lastmem=after;
		end
	end
end